# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionComment(Question):
    """SurveyJS Comment (Long Text / Multi-line) question."""

    @property
    def max_length(self):
        return self.raw.get('maxLength', 0)

    @property
    def rows(self):
        """Number of visible rows in the text area."""
        return self.raw.get('rows', 4)

    @property
    def auto_grow(self):
        return self.raw.get('autoGrow', False)

    @property
    def allow_resize(self):
        return self.raw.get('allowResize', True)



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionTagbox(Question):
    """SurveyJS Tag Box (multi-select dropdown) question.

    Value is typically a list of selected values.
    """

    @property
    def choices(self):
        return self.raw.get('choices', [])

    @property
    def choices_values(self):
        result = []
        for choice in self.choices:
            if isinstance(choice, dict):
                result.append({
                    'value': choice.get('value'),
                    'text': choice.get('text', str(choice.get('value', '')))
                })
            else:
                result.append({'value': choice, 'text': str(choice)})
        return result

    @property
    def value_texts(self):
        """Get the display texts for the currently selected values."""
        val = self.value
        if not val or not isinstance(val, list):
            return []
        texts = []
        choice_map = {c['value']: c['text'] for c in self.choices_values}
        for v in val:
            texts.append(choice_map.get(v, str(v)))
        return texts

    @property
    def has_other(self):
        return self.raw.get('showOtherItem', self.raw.get('hasOther', False))

    @property
    def search_enabled(self):
        return self.raw.get('searchEnabled', True)

    @property
    def max_selected_choices(self):
        return self.raw.get('maxSelectedChoices', 0)



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .element import Element

# TODO
# Move some properties from Element to Question, like is_required, validators, etc.,
# so we can have common question properties (is_required, validators, etc.) in one place.

class Question(Element):
    """
    Base class for all SurveyJS Question types.
    """

    @property
    def is_question(self):
        """Whether this is a question (has user-submitted values).
        Layout elements (panel, html, image) override this to return False."""
        return True

    @property
    def is_input(self):
        """Whether this is a question (has user-submitted values).
        Layout elements (panel, html, image) override this to return False."""
        return self.is_question



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from collections import OrderedDict

from .layout import Layout


class QuestionPanel(Layout):
    """SurveyJS Panel (layout container) question.

    Panels group questions together. They are not input elements themselves.
    """

    @property
    def panel_title(self):
        return self.raw.get('title', '')

    @property
    def state(self):
        """Panel state: 'default', 'collapsed', or 'expanded'."""
        return self.raw.get('state', 'default')

    @property
    def inner_indent(self):
        return self.raw.get('innerIndent', 0)

    def load_data(self, data, is_form=False):
        """Load nested question data."""
        for element in self._layout_elements:
            if 'type' in element:
                element_obj = self.survey.get_element_object(element)
                if element_obj:
                    element_obj.load(
                        element_owner=self.element_owner,
                        parent=self,
                        data=data,
                        is_form=is_form,
                    )



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the file question type."""

import unittest

from surveyjs.elements.file import QuestionFile
from tests.utils import load_creator, load_form


class TestQuestionFile(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['resume']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionFile)

    def test_type(self):
        self.assertEqual(self.q.type, 'file')

    def test_title(self):
        self.assertEqual(self.q.title, 'Upload Resume')

    def test_store_data_as_text(self):
        self.assertTrue(self.q.store_data_as_text)

    def test_allow_multiple(self):
        self.assertTrue(self.q.allow_multiple)

    def test_max_size(self):
        self.assertEqual(self.q.max_size, 5242880)

    def test_accepted_types(self):
        self.assertEqual(self.q.accepted_types, '.pdf,.doc,.docx')


class TestFileForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.questions['resume']

    def test_value_is_list(self):
        self.assertIsInstance(self.q.value, list)

    def test_files(self):
        files = self.q.files
        self.assertEqual(len(files), 2)

    def test_file_count(self):
        self.assertEqual(self.q.file_count, 2)

    def test_file_names(self):
        files = self.q.files
        names = [f['name'] for f in files]
        self.assertIn('resume.pdf', names)
        self.assertIn('cover_letter.docx', names)

    def test_file_types(self):
        files = self.q.files
        types = [f['type'] for f in files]
        self.assertIn('application/pdf', types)


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .layout import Layout


class QuestionImage(Layout):
    """SurveyJS Image question.

    Displays a static image. Not an input element.
    """

    @property
    def is_question(self):
        """Image elements are display-only, not input questions."""
        return False

    @property
    def image_link(self):
        """Get the image URL."""
        return self.raw.get('imageLink', '')

    @property
    def image_fit(self):
        """Image fit: 'contain', 'cover', 'fill', 'none'."""
        return self.raw.get('imageFit', 'contain')

    @property
    def image_height(self):
        return self.raw.get('imageHeight', 150)

    @property
    def image_width(self):
        return self.raw.get('imageWidth', 200)

    @property
    def alt_text(self):
        return self.raw.get('altText', '')

    @property
    def content_mode(self):
        """Content mode: 'image', 'video', 'youtube'."""
        return self.raw.get('contentMode', 'image')



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionMatrixdropdown(Question):
    """SurveyJS Multi-Select Matrix (matrixdropdown) question.

    Value is a dict mapping row names to dicts of column-name -> value.
    Example: {"row1": {"col1": "val1", "col2": "val2"}}
    """

    @property
    def columns(self):
        """Get the matrix columns."""
        return self.raw.get('columns', [])

    @property
    def rows(self):
        """Get the matrix rows."""
        return self.raw.get('rows', [])

    @property
    def cell_type(self):
        """Default cell type for all columns: 'dropdown', 'checkbox',
        'radiogroup', 'text', 'comment', 'boolean', 'expression', 'rating'."""
        return self.raw.get('cellType', 'dropdown')

    @property
    def choices(self):
        """Default choices for all columns."""
        return self.raw.get('choices', [])

    @property
    def row_count(self):
        return len(self.rows)

    @property
    def column_count(self):
        return len(self.columns)

    def get_row_value(self, row_name):
        """Get the values dict for a specific row."""
        val = self.value
        if val and isinstance(val, dict):
            return val.get(row_name, {})
        return {}

    def get_cell_value(self, row_name, column_name):
        """Get a specific cell value."""
        row_val = self.get_row_value(row_name)
        if isinstance(row_val, dict):
            return row_val.get(column_name)
        return None



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

import json
import logging
from collections import OrderedDict

from surveyjs import SurveyCreator

logger = logging.getLogger(__name__)


class SurveyForm:
    """
    Represents a filled-in SurveyJS form (submission data).

    Parses the form submission JSON against a SurveyCreator schema and creates
    Question (Element) objects with values populated from the submission.
    """

    def __init__(
        self,
        form_json,
        creator=None,
        creator_schema_json=None,
        lang="en",
        **kwargs
    ):
        """
        @param form_json: SurveyJS Form submission JSON (str or dict)
        @param creator: A SurveyCreator instance
        @param creator_schema_json: SurveyCreator schema JSON (str or dict)
            Alternative to providing a SurveyCreator instance.
        @param lang: Language code for translations (default: 'en')
        """

        if creator and creator_schema_json:
            raise Exception(
                "Constructor accepts either creator or creator_schema_json, not both."
            )

        if isinstance(form_json, dict):
            self.form = form_json
        else:
            self.form = json.loads(form_json)

        self.creator = creator
        self.creator_schema_json = creator_schema_json

        if self.creator is None and self.creator_schema_json:
            self.creator = SurveyCreator(self.creator_schema_json)

        if self.creator:
            if not isinstance(self.creator, SurveyCreator):
                raise TypeError("creator must be a SurveyCreator instance")
        elif self.creator_schema_json:
            assert isinstance(self.creator_schema_json, str)

        else:
            raise Exception(
                "Provide either the argument: creator or creator_schema_json."
            )

        self.lang = lang
        # defaults to English (en) date/time format
        self.date_format = kwargs.get('date_format', '%m/%d/%Y')
        self.time_format = kwargs.get('time_format', '%H:%M:%S')

        # All elements (question + layout) keyed by name
        self.elements = OrderedDict()

        # Questions (fields) keyed by name
        self.questions = OrderedDict()

        # Elements keyed by id
        self.element_ids = {}

        # Load elements from survey schema + populate values from form data
        self.load_elements()

        # Create attribute-style accessor
        self._data = FormData(self)

    def set_creator_by_creator_schema_json(self):
        self.creator = SurveyCreator(
            self.creator_schema_json,
            language=self.lang,
        )

    # TODO?
    # @property
    # def elements(self):
    #     return self._elements

    # @elements.setter
    # def elements(self, elements):
    #     if isinstance(elements, OrderedDict):
    #         self._elements = elements
    #     else:
    #         self._elements = OrderedDict(elements)

    @property
    def data(self):
        """Attribute-style access to input questions."""
        return self._data

    @property
    def components(self):
        """ Alias for elements, including both question and layout elements."""
        return self.elements

    @property
    def input_components(self):
        """ Alias for questions."""
        return self.questions

    def load_elements(self):
        """Load elements from the survey schema and populate values from
        form submission data."""
        for key, creator_element in self.creator.elements.items():
            # Create a new element object (don't affect the Survey's element)
            element_obj = self.creator.get_element_object(creator_element.raw)
            element_obj.load(
                element_owner=self,
                parent=None,
                data=self.form,
                is_form=True,
            )
            self.elements[key] = element_obj

            if element_obj.id:
                self.element_ids[element_obj.id] = element_obj

    def get_question_by_name(self, name):
        """Get a question by its name."""
        return self.questions.get(name)

    def get_value(self, name):
        """Get the value of a question by name."""
        question = self.questions.get(name)
        if question:
            return question.value
        return None


class FormData:
    """Provides attribute-style access to form input questions.

    Example:
        form.data.firstName.value  # => 'Bob'
    """

    def __init__(self, form):
        self._form = form

    def __getattr__(self, key):
        if key.startswith('_'):
            return super().__getattribute__(key)
        return self._form.questions.get(key)



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionSignaturepad(Question):
    """SurveyJS Signature Pad question.

    Value is typically a base64-encoded image string.
    """

    @property
    def signature_width(self):
        return self.raw.get('signatureWidth', 300)

    @property
    def signature_height(self):
        return self.raw.get('signatureHeight', 200)

    @property
    def pen_color(self):
        return self.raw.get('penColor', '#1ab394')

    @property
    def background_color(self):
        return self.raw.get('backgroundColor', '#ffffff')

    @property
    def data_format(self):
        """'png', 'jpeg', or 'svg'."""
        return self.raw.get('dataFormat', 'png')

    @property
    def allow_clear(self):
        return self.raw.get('allowClear', True)

    @property
    def is_base64(self):
        """Check if the value is a base64-encoded string."""
        val = self.value
        if val and isinstance(val, str):
            return val.startswith('data:image/')
        return False



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionDropdown(Question):
    """SurveyJS Dropdown (single-select) question."""

    @property
    def choices(self):
        return self.raw.get('choices', [])

    @property
    def choices_values(self):
        result = []
        for choice in self.choices:
            if isinstance(choice, dict):
                result.append({
                    'value': choice.get('value'),
                    'text': choice.get('text', str(choice.get('value', '')))
                })
            else:
                result.append({'value': choice, 'text': str(choice)})
        return result

    @property
    def value_text(self):
        """Get the display text for the current value."""
        val = self.value
        for choice in self.choices_values:
            if choice['value'] == val:
                return choice['text']
        return str(val) if val is not None else None

    @property
    def has_other(self):
        return self.raw.get('showOtherItem', self.raw.get('hasOther', False))

    @property
    def other_text(self):
        return self.raw.get('otherText', 'Other')

    @property
    def choices_by_url(self):
        """Get choices loaded from a URL configuration."""
        return self.raw.get('choicesByUrl', None)

    @property
    def search_enabled(self):
        return self.raw.get('searchEnabled', True)

    @property
    def placeholder_text(self):
        return self.raw.get('placeholder', '')

    @property
    def allow_clear(self):
        return self.raw.get('allowClear', True)



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionPaneldynamic(Question):
    """SurveyJS Dynamic Panel question.

    Allows users to add/remove groups of questions (panels).
    Value is a list of dicts, each dict representing one panel instance.
    """

    @property
    def template_elements(self):
        """Template elements that repeat for each panel."""
        return self.raw.get('templateElements', self.raw.get('elements', []))

    @property
    def panel_count(self):
        """Initial panel count."""
        return self.raw.get('panelCount', 1)

    @property
    def min_panel_count(self):
        return self.raw.get('minPanelCount', 0)

    @property
    def max_panel_count(self):
        return self.raw.get('maxPanelCount', None)

    @property
    def panel_add_text(self):
        return self.raw.get('panelAddText', 'Add New')

    @property
    def panel_remove_text(self):
        return self.raw.get('panelRemoveText', 'Remove')

    @property
    def allow_add_panel(self):
        return self.raw.get('allowAddPanel', True)

    @property
    def allow_remove_panel(self):
        return self.raw.get('allowRemovePanel', True)

    @property
    def template_title(self):
        return self.raw.get('templateTitle', '')

    @property
    def panels_data(self):
        """Get the value as a list of dicts (panel instances)."""
        val = self.value
        if isinstance(val, list):
            return val
        return []

    @property
    def actual_panel_count(self):
        """Number of actual panel instances in the data."""
        return len(self.panels_data)

    def get_panel_value(self, panel_index, field_name):
        """Get a specific field value from a panel instance.

        Args:
            panel_index: Zero-based index of the panel instance.
            field_name: Name of the field within the panel.

        Returns:
            The value of the field, or None.
        """
        data = self.panels_data
        if panel_index < len(data) and isinstance(data[panel_index], dict):
            return data[panel_index].get(field_name)
        return None



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

import uuid
from collections import OrderedDict

# TODO
# 1. Make a Question class that inherits from Element and all questions inherit from that,
# so we can have common question properties (is_required, validators, etc.) in one place.
# 2. Make a Layout class that inherits from Element and all layout classes (panel) inherit from that.

class Element:
    """
    Base class for all SurveyJS element types.

    Holds the raw JSON element data, provides access to common properties
    (name, type, title, etc.) and handles value loading from form data.
    """

    _none_value = None

    def __init__(self, raw, survey, **kwargs):
        """
        @param raw: The raw JSON element dict from the survey schema
        @param survey: The Survey instance that owns this element
        """
        self.raw = raw
        self.survey = survey

        self._parent = None
        self._element_owner = None

        # Child elements (for panels and other containers)
        self._elements = OrderedDict()

        # Path tracking
        self.survey_path = []
        self.survey_input_path = []

        # ID
        self._id = self.raw.get('id', str(uuid.uuid4()))

        # Submission data at this level
        self.form = {}
        self._all_data = {}

        # i18n
        self.language = kwargs.get('language', 'en')
        self.i18n = kwargs.get('i18n', {})

        self.default_value = self.raw.get('defaultValue')

    def load(self, element_owner, parent=None, data=None, is_form=False):
        """Load the element into its owner (Survey or Form) and populate
        data from form submission if applicable."""
        self.element_owner = element_owner

        if parent:
            self.parent = parent

        self._all_data = data or {}
        self.load_data(data, is_form=is_form)

    def load_data(self, data, is_form=False):
        """Load value from form submission data."""
        if self.is_question and data:
            try:
                self.value = data[self.name]
            except KeyError:
                # Use default value if available
                pass

    # --- Properties ---

    @property
    def is_question(self):
        """Whether this is a question (has user-submitted values).
        Layout elements (panel, html, image) override this to return False."""
        return None

    @property
    def elements(self):
        return self._elements

    @elements.setter
    def elements(self, elements):
        if isinstance(elements, OrderedDict):
            self._elements = elements
        else:
            self._elements = OrderedDict(elements)

    @property
    def id(self):
        return self._id

    @property
    def name(self):
        """The unique name/key of the element."""
        return self.raw.get('name', '')

    @property
    def type(self):
        """The SurveyJS element type."""
        return self.raw.get('type', '')

    @property
    def title(self):
        """The display title of the element."""
        title = self.raw.get('title')
        if isinstance(title, dict):
            title = title.get(self.language) or title.get(next(iter(title), None)) or self.name
        if not title:
            title = self.name
        if self.i18n.get(self.language):
            return self.i18n[self.language].get(title, title)
        return title

    @property
    def label(self):
        """Alias for title."""
        return self.title

    @property
    def description(self):
        """The element description/help text."""
        return self.raw.get('description', '')

    @property
    def is_required(self):
        """Whether the element is required."""
        return self.raw.get('isRequired', False)

    @property
    def is_visible(self):
        """Whether the element is visible."""
        return self.raw.get('visible', True)

    @property
    def visible_if(self):
        """The visibleIf expression."""
        return self.raw.get('visibleIf', '')

    @property
    def enable_if(self):
        """The enableIf expression."""
        return self.raw.get('enableIf', '')

    @property
    def read_only(self):
        """Whether the question is read-only."""
        return self.raw.get('readOnly', False)

    @property
    def validators(self):
        """The list of validators."""
        return self.raw.get('validators', [])

    @property
    def input_type(self):
        """The input type (for text questions)."""
        return self.raw.get('inputType', '')

    @property
    def placeholder(self):
        """The placeholder text."""
        return self.raw.get('placeholder', '')

    @property
    def start_with_new_line(self):
        """Whether the element should start on a new line (for layout purposes).
        Defaults to True if not specified.

        Disable this property if you want to render the current question on the
        same line or row with the previous question or panel."""
        return self.raw.get('startWithNewLine', True)

    @property
    def elements_on_same_line(self):
        """Return all elements on the same visual line as self, or []
        if self is alone on its line.

        A line group starts with an element whose start_with_new_line is
        True (or the very first element) and includes all consecutive
        followers with start_with_new_line False."""
        if not self._element_owner:
            return []

        owner_els = list(self._element_owner.elements.values())

        # Find the start of self's line group (walk back to the nearest
        # element with start_with_new_line True, or index 0).
        self_idx = owner_els.index(self)
        start = self_idx
        while start > 0 and not owner_els[start].start_with_new_line:
            start -= 1

        # Collect the group: start element + all consecutive followers
        # with start_with_new_line False.
        group = [owner_els[start]]
        for el in owner_els[start + 1:]:
            if el.start_with_new_line:
                break
            group.append(el)

        return group if len(group) > 1 else []

    @property
    def parent(self):
        return self._parent

    @parent.setter
    def parent(self, parent):
        if parent:
            self._parent = parent
            self._parent.elements[self.name] = self

    @property
    def element_owner(self):
        """The element's owner (Survey or Form)."""
        return self._element_owner

    @element_owner.setter
    def element_owner(self, element_owner):
        self._element_owner = element_owner
        if self.is_question:
            self._element_owner.questions[self.name] = self

    # --- Value ---

    @property
    def value(self):
        """Get the question's value from form data."""
        val = self.form.get('value', self._none_value)
        if val is self._none_value:
            return self.default_value
        return val

    @value.setter
    def value(self, value):
        self.form['value'] = value

    @property
    def raw_value(self):
        """Get the raw (unprocessed) value."""
        return self.form.get('raw_value', self.value)

    @raw_value.setter
    def raw_value(self, value):
        self.form['raw_value'] = value

    # -- custom properties ---
    @property
    def custom_properties(self):
        """The list of customProperties.

        Documentation:
        https://surveyjs.io/form-library/documentation/customize-question-types/add-custom-properties-to-a-form

        E.g. property type itemvalues:
        - Customized text inputs for entering value-text pairs
        - Use this type for arrays of objects with the following structure:
          { value: any, text: string }.
          For example, Dropdown, Checkboxes, and Radio Button Group questions use this type for the choices property.
        """
        return self.raw.get('customProperties', [])

    # --- Representation ---

    def __repr__(self):
        return '<%s name=%s>' % (self.__class__.__name__, self.name)

    def to_dict(self):
        """Return a dictionary representation of the element (question)."""
        return {
            'name': self.name,
            'type': self.type,
            'title': self.title,
            'value': self.value,
        }



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the text question type."""

import unittest

from tests.utils import load_creator, load_form


class TestCreatorStartWithNewLine(unittest.TestCase):

    def setUp(self):
        self.creator = load_creator()

    def test_start_with_new_line_default_true(self):
        """startWithNewLine defaults to True when not specified."""
        self.assertTrue(self.creator.questions['firstName'].start_with_new_line)

    def test_start_with_new_line_false(self):
        """startWithNewLine is False when explicitly set."""
        self.assertFalse(self.creator.questions['age'].start_with_new_line)
        self.assertFalse(self.creator.questions['birthDate'].start_with_new_line)

    def test_elements_on_same_line_returns_false_start_with_new_line(self):
        """elements_on_same_line returns sibling elements where startWithNewLine is False."""
        elements_same_line = self.creator.questions['firstName'].elements_on_same_line
        names_same_line = [el.name for el in elements_same_line]
        self.assertEqual(len(elements_same_line), 3)
        self.assertIn('firstName', names_same_line)
        self.assertIn('age', names_same_line)
        self.assertIn('birthDate', names_same_line)

        elements_same_line = self.creator.questions['age'].elements_on_same_line
        names_same_line = [el.name for el in elements_same_line]
        self.assertEqual(len(elements_same_line), 3)
        self.assertIn('firstName', names_same_line)
        self.assertIn('age', names_same_line)
        self.assertIn('birthDate', names_same_line)

        elements_same_line = self.creator.questions['birthDate'].elements_on_same_line
        names_same_line = [el.name for el in elements_same_line]
        self.assertEqual(len(elements_same_line), 3)
        self.assertIn('firstName', names_same_line)
        self.assertIn('age', names_same_line)
        self.assertIn('birthDate', names_same_line)

    def test_elements_on_same_line_excludes_default_start_with_new_line(self):
        """elements_on_same_line does not include elements where startWithNewLine defaults to True."""
        elements_same_line = self.creator.questions['email'].elements_on_same_line
        self.assertEqual(elements_same_line, [])
        elements_same_line = self.creator.questions['appointmentTime'].elements_on_same_line
        self.assertEqual(elements_same_line, [])
        names_same_line = [el.name for el in elements_same_line]
        self.assertNotIn('firstName', names_same_line)
        self.assertNotIn('age', names_same_line)
        self.assertNotIn('birthDate', names_same_line)
        self.assertNotIn('email', names_same_line)


class TestFormStartWithNewLine(unittest.TestCase):

    def setUp(self):
        self.form = load_form()

    def test_start_with_new_line_default_true(self):
        """startWithNewLine defaults to True when not specified."""
        self.assertTrue(self.form.questions['firstName'].start_with_new_line)

    def test_start_with_new_line_false(self):
        """startWithNewLine is False when explicitly set."""
        self.assertFalse(self.form.questions['age'].start_with_new_line)

    def test_elements_on_same_line_returns_false_start_with_new_line(self):
        """elements_on_same_line returns sibling elements where startWithNewLine is False."""
        elements_same_line = self.form.questions['firstName'].elements_on_same_line
        names_same_line = [el.name for el in elements_same_line]
        self.assertEqual(len(elements_same_line), 3)
        self.assertIn('firstName', names_same_line)
        self.assertIn('age', names_same_line)
        self.assertIn('birthDate', names_same_line)

        elements_same_line = self.form.questions['age'].elements_on_same_line
        names_same_line = [el.name for el in elements_same_line]
        self.assertEqual(len(elements_same_line), 3)
        self.assertIn('firstName', names_same_line)
        self.assertIn('age', names_same_line)
        self.assertIn('birthDate', names_same_line)

        elements_same_line = self.form.questions['birthDate'].elements_on_same_line
        names_same_line = [el.name for el in elements_same_line]
        self.assertEqual(len(elements_same_line), 3)
        self.assertIn('firstName', names_same_line)
        self.assertIn('age', names_same_line)
        self.assertIn('birthDate', names_same_line)

    def test_elements_on_same_line_excludes_default_start_with_new_line(self):
        """elements_on_same_line does not include elements where startWithNewLine defaults to True."""
        elements_same_line = self.form.questions['email'].elements_on_same_line
        names_same_line = [el.name for el in elements_same_line]
        self.assertNotIn('firstName', names_same_line)
        self.assertNotIn('age', names_same_line)
        self.assertNotIn('birthDate', names_same_line)
        self.assertNotIn('email', names_same_line)


    def test_elements_on_same_line_count(self):
        """elements_on_same_line returns exactly the elements with startWithNewLine False."""
        elements = self.form.questions['age'].elements_on_same_line
        self.assertEqual(len(elements), 3)

if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionMatrix(Question):
    """SurveyJS Single-Select Matrix question.

    Value is a dict mapping row names to selected column values.
    Example: {"row1": "col2", "row2": "col1"}
    """

    @property
    def columns(self):
        """Get the matrix columns."""
        return self.raw.get('columns', [])

    @property
    def column_values(self):
        """Normalized column values."""
        result = []
        for col in self.columns:
            if isinstance(col, dict):
                result.append({
                    'value': col.get('value'),
                    'text': col.get('text', str(col.get('value', '')))
                })
            else:
                result.append({'value': col, 'text': str(col)})
        return result

    @property
    def rows(self):
        """Get the matrix rows."""
        return self.raw.get('rows', [])

    @property
    def row_values(self):
        """Normalized row values."""
        result = []
        for row in self.rows:
            if isinstance(row, dict):
                result.append({
                    'value': row.get('value'),
                    'text': row.get('text', str(row.get('value', '')))
                })
            else:
                result.append({'value': row, 'text': str(row)})
        return result

    @property
    def is_all_row_required(self):
        return self.raw.get('isAllRowRequired', False)

    def get_row_value(self, row_name):
        """Get the selected value for a specific row."""
        val = self.value
        if val and isinstance(val, dict):
            return val.get(row_name)
        return None



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the boolean question type."""

import unittest

from surveyjs.elements.boolean import QuestionBoolean
from tests.utils import load_creator, load_form


class TestQuestionBoolean(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['agreeTerms']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionBoolean)

    def test_type(self):
        self.assertEqual(self.q.type, 'boolean')

    def test_title(self):
        self.assertEqual(self.q.title, 'I agree to the Terms of Service')

    def test_label_true(self):
        self.assertEqual(self.q.label_true, 'Yes')

    def test_label_false(self):
        self.assertEqual(self.q.label_false, 'No')

    def test_value_true(self):
        self.assertEqual(self.q.value_true, 'agreed')

    def test_value_false(self):
        self.assertEqual(self.q.value_false, 'declined')

    def test_render_as(self):
        self.assertEqual(self.q.render_as, 'checkbox')


class TestBooleanForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.questions['agreeTerms']

    def test_value(self):
        self.assertEqual(self.q.value, 'agreed')

    def test_to_bool_custom_true(self):
        self.assertTrue(self.q.to_bool())


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionMatrixdynamic(Question):
    """SurveyJS Dynamic Matrix (matrixdynamic) question.

    Value is a list of dicts, where each dict maps column names to values.
    Example: [{"col1": "val1", "col2": "val2"}, {"col1": "val3", "col2": "val4"}]
    """

    @property
    def columns(self):
        """Get the matrix columns."""
        return self.raw.get('columns', [])

    @property
    def row_count(self):
        """Initial/current number of rows."""
        return self.raw.get('rowCount', 2)

    @property
    def min_row_count(self):
        return self.raw.get('minRowCount', 0)

    @property
    def max_row_count(self):
        return self.raw.get('maxRowCount', 1000)

    @property
    def add_row_text(self):
        return self.raw.get('addRowText', '')

    @property
    def remove_row_text(self):
        return self.raw.get('removeRowText', '')

    @property
    def cell_type(self):
        return self.raw.get('cellType', 'dropdown')

    @property
    def choices(self):
        return self.raw.get('choices', [])

    @property
    def rows_data(self):
        """Get the list of row data dicts."""
        val = self.value
        if val and isinstance(val, list):
            return val
        return []

    @property
    def actual_row_count(self):
        """Number of rows in submitted data."""
        return len(self.rows_data)

    def get_cell_value(self, row_index, column_name):
        """Get a specific cell value by row index and column name."""
        data = self.rows_data
        if 0 <= row_index < len(data):
            return data[row_index].get(column_name)
        return None



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionFile(Question):
    """SurveyJS File Upload question.

    Value is typically a list of file metadata objects or base64 data.
    """

    @property
    def store_data_as_text(self):
        """Whether files are stored as base64 text."""
        return self.raw.get('storeDataAsText', True)

    @property
    def allow_multiple(self):
        return self.raw.get('allowMultiple', False)

    @property
    def max_size(self):
        """Maximum file size in bytes (0 = unlimited)."""
        return self.raw.get('maxSize', 0)

    @property
    def accepted_types(self):
        """Accepted file types (MIME types or extensions)."""
        return self.raw.get('acceptedTypes', '')

    @property
    def allow_camera_access(self):
        return self.raw.get('allowCameraAccess', False)

    @property
    def source_type(self):
        return self.raw.get('sourceType', 'file')

    @property
    def files(self):
        """Get the list of file objects from the value."""
        val = self.value
        if val is None:
            return []
        if isinstance(val, list):
            return val
        return [val]

    @property
    def file_count(self):
        """Number of uploaded files."""
        return len(self.files)



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionBoolean(Question):
    """SurveyJS Boolean (Yes/No) question.

    Value is typically True/False.
    """

    @property
    def label_true(self):
        return self.raw.get('labelTrue', 'Yes')

    @property
    def label_false(self):
        return self.raw.get('labelFalse', 'No')

    @property
    def value_true(self):
        return self.raw.get('valueTrue', True)

    @property
    def value_false(self):
        return self.raw.get('valueFalse', False)

    @property
    def render_as(self):
        """Render mode: 'default' (toggle) or 'checkbox' or 'radio'."""
        return self.raw.get('renderAs', 'default')

    def to_bool(self):
        """Convert the value to a Python bool."""
        val = self.value
        if val is None:
            return None
        if isinstance(val, bool):
            return val
        if val == self.value_true:
            return True
        if val == self.value_false:
            return False
        # Fallback
        return bool(val)



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the image question type."""

import unittest

from surveyjs.elements.image import QuestionImage
from tests.utils import load_creator, load_form


class TestQuestionImage(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.elements['thankYouImage']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionImage)

    def test_type(self):
        self.assertEqual(self.q.type, 'image')

    # TODO
    # def test_is_question(self):
    #     self.assertFalse(self.q.is_question)

    def test_is_input(self):
        self.assertFalse(self.q.is_input)

    def test_image_link(self):
        self.assertEqual(self.q.image_link, 'https://example.com/thankyou.png')

    def test_image_fit(self):
        self.assertEqual(self.q.image_fit, 'contain')

    def test_image_height(self):
        self.assertEqual(self.q.image_height, 300)

    def test_image_width(self):
        self.assertEqual(self.q.image_width, 400)

    def test_alt_text(self):
        self.assertEqual(self.q.alt_text, 'Thank you')

    def test_content_mode(self):
        self.assertEqual(self.q.content_mode, 'image')

    def test_not_in_questions(self):
        self.assertNotIn('thankYouImage', self.survey.questions)

    def test_in_elements(self):
        self.assertIn('thankYouImage', self.survey.elements)


class TestImageForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()

    def test_not_in_questions(self):
        self.assertNotIn('thankYouImage', self.form.questions)


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the imagepicker question type."""

import unittest

from surveyjs.elements.imagepicker import QuestionImagepicker
from tests.utils import load_creator, load_form


class TestQuestionImagepicker(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['favoritePet']

    def test_class_type(self):

        self.assertIsInstance(self.q, QuestionImagepicker)

    def test_type(self):
        self.assertEqual(self.q.type, 'imagepicker')

    def test_title(self):
        self.assertEqual(self.q.title, 'Favorite Pet')

    def test_choices(self):
        self.assertEqual(len(self.q.choices), 3)

    def test_multi_select(self):
        self.assertFalse(self.q.multi_select)

    def test_show_label(self):
        self.assertTrue(self.q.show_label)

    def test_image_fit(self):
        self.assertEqual(self.q.image_fit, 'cover')

    def test_image_height(self):
        self.assertEqual(self.q.image_height, 150)

    def test_image_width(self):
        self.assertEqual(self.q.image_width, 200)

    def test_content_mode(self):
        self.assertEqual(self.q.content_mode, 'image')


class TestImagepickerForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.questions['favoritePet']

    def test_value(self):
        self.assertEqual(self.q.value, 'cat')


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the radiogroup question type."""

import unittest

from surveyjs.elements.radiogroup import QuestionRadiogroup
from tests.utils import load_creator, load_form


class TestQuestionRadiogroup(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['favoriteColor']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionRadiogroup)

    def test_type(self):
        self.assertEqual(self.q.type, 'radiogroup')

    def test_title(self):
        self.assertEqual(self.q.title, 'Favorite Color')

    def test_choices(self):
        self.assertEqual(len(self.q.choices), 4)

    def test_choices_values(self):
        values = self.q.choices_values
        self.assertEqual(len(values), 4)
        self.assertEqual(values[0]['value'], 'red')
        self.assertEqual(values[0]['text'], 'Red')

    def test_has_other(self):
        self.assertTrue(self.q.has_other)

    def test_has_none(self):
        self.assertFalse(self.q.has_none)

    def test_choices_order(self):
        self.assertEqual(self.q.choices_order, 'random')

    def test_col_count(self):
        self.assertEqual(self.q.col_count, 2)

    def test_is_required(self):
        self.assertTrue(self.q.is_required)


class TestRadiogroupForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.questions['favoriteColor']

    def test_value(self):
        self.assertEqual(self.q.value, 'blue')

    def test_value_text(self):
        self.assertEqual(self.q.value_text, 'Blue')


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the matrixdropdown question type."""

import unittest

from surveyjs.elements.matrixdropdown import QuestionMatrixdropdown
from tests.utils import load_creator, load_form


class TestQuestionMatrixdropdown(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['budgetMatrix']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionMatrixdropdown)

    def test_type(self):
        self.assertEqual(self.q.type, 'matrixdropdown')

    def test_title(self):
        self.assertEqual(self.q.title, 'Annual Budget')

    def test_columns(self):
        self.assertEqual(len(self.q.columns), 3)

    def test_column_names(self):
        col_names = [c['name'] for c in self.q.columns]
        self.assertIn('q1', col_names)
        self.assertIn('q2', col_names)
        self.assertIn('status', col_names)

    def test_rows(self):
        self.assertEqual(len(self.q.rows), 2)

    def test_cell_type(self):
        self.assertEqual(self.q.cell_type, 'text')

    def test_choices(self):
        self.assertEqual(len(self.q.choices), 3)


class TestMatrixDropdownForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.questions['budgetMatrix']

    def test_value_is_dict(self):
        self.assertIsInstance(self.q.value, dict)

    def test_get_row_value(self):
        row = self.q.get_row_value('marketing')
        self.assertIsInstance(row, dict)
        self.assertEqual(row['q1'], '10000')
        self.assertEqual(row['status'], 'approved')

    def test_get_cell_value(self):
        self.assertEqual(self.q.get_cell_value('marketing', 'q1'), '10000')
        self.assertEqual(self.q.get_cell_value('development', 'q2'), '55000')
        self.assertEqual(self.q.get_cell_value('development', 'status'), 'planned')

    def test_get_cell_value_nonexistent(self):
        self.assertIsNone(self.q.get_cell_value('marketing', 'nonexistent'))
        self.assertIsNone(self.q.get_cell_value('nonexistent', 'q1'))


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .element import Element


class Layout(Element):
    """
    Base class for all SurveyJS layout types.
    """

    @property
    def is_question(self):
        """Panels are layout elements, not input questions."""
        return False

    @property
    def is_input(self):
        """Panels are layout elements, not input questions."""
        return False

    @property
    def _layout_elements(self):
        """Get the raw nested elements."""
        return self.raw.get('elements', [])



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the expression question type."""

import unittest

from surveyjs.elements.expression import QuestionExpression
from tests.utils import load_creator, load_form


class TestQuestionExpression(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['fullName']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionExpression)

    def test_type(self):
        self.assertEqual(self.q.type, 'expression')

    def test_title(self):
        self.assertEqual(self.q.title, 'Full Name')

    def test_expression(self):
        self.assertEqual(self.q.expression, "{firstName} + ' Smith'")

    def test_display_style(self):
        self.assertEqual(self.q.display_style, 'none')


class TestExpressionForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.questions['fullName']

    def test_value(self):
        self.assertEqual(self.q.value, 'Alice Smith')

    def test_to_number_returns_none_for_string(self):
        self.assertIsNone(self.q.to_number())


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the checkbox question type."""

import unittest

from surveyjs.elements.checkbox import QuestionCheckbox
from tests.utils import load_creator, load_form


class TestQuestionCheckbox(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['hobbies']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionCheckbox)

    def test_type(self):
        self.assertEqual(self.q.type, 'checkbox')

    def test_title(self):
        self.assertEqual(self.q.title, 'Hobbies')

    def test_choices(self):
        self.assertEqual(len(self.q.choices), 5)

    def test_choices_values(self):
        values = self.q.choices_values
        self.assertEqual(len(values), 5)
        val_names = [v['value'] for v in values]
        self.assertIn('reading', val_names)
        self.assertIn('sports', val_names)

    def test_has_other(self):
        self.assertTrue(self.q.has_other)

    def test_has_none(self):
        self.assertTrue(self.q.has_none)

    def test_has_select_all(self):
        self.assertTrue(self.q.has_select_all)

    def test_max_selected_choices(self):
        self.assertEqual(self.q.max_selected_choices, 3)

    def test_min_selected_choices(self):
        self.assertEqual(self.q.min_selected_choices, 1)

    def test_col_count(self):
        self.assertEqual(self.q.col_count, 3)


class TestCheckboxForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.questions['hobbies']

    def test_value_is_list(self):
        self.assertIsInstance(self.q.value, list)

    def test_value_contents(self):
        self.assertEqual(self.q.value, ['reading', 'music', 'travel'])

    def test_value_texts(self):
        texts = self.q.value_texts
        self.assertIn('Reading', texts)
        self.assertIn('Music', texts)
        self.assertIn('Travel', texts)

    def test_value_texts_count(self):
        self.assertEqual(len(self.q.value_texts), 3)


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the comment question type."""

import unittest

from surveyjs.elements.comment import QuestionComment
from tests.utils import load_creator, load_form


class TestQuestionComment(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['bio']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionComment)

    def test_type(self):
        self.assertEqual(self.q.type, 'comment')

    def test_title(self):
        self.assertEqual(self.q.title, 'Biography')

    def test_description(self):
        self.assertEqual(self.q.description, 'Tell us about yourself')

    def test_max_length(self):
        self.assertEqual(self.q.max_length, 500)

    def test_rows(self):
        self.assertEqual(self.q.rows, 4)

    def test_auto_grow(self):
        self.assertTrue(self.q.auto_grow)

    def test_allow_resize(self):
        self.assertFalse(self.q.allow_resize)

    def test_is_input(self):
        self.assertTrue(self.q.is_input)


class TestCommentForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.questions['bio']

    def test_value(self):
        self.assertIn('Software developer', self.q.value)

    def test_value_string(self):
        self.assertIsInstance(self.q.value, str)


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionRadiogroup(Question):
    """SurveyJS Radio Button Group question."""

    @property
    def choices(self):
        """Get the list of choices."""
        return self.raw.get('choices', [])

    @property
    def choices_values(self):
        """Get normalized choice values as a list of dicts with 'value' and 'text'."""
        result = []
        for choice in self.choices:
            if isinstance(choice, dict):
                result.append({
                    'value': choice.get('value'),
                    'text': choice.get('text', str(choice.get('value', '')))
                })
            else:
                result.append({'value': choice, 'text': str(choice)})
        return result

    @property
    def value_text(self):
        """Get the display text for the current value."""
        val = self.value
        for choice in self.choices_values:
            if choice['value'] == val:
                return choice['text']
        return str(val) if val is not None else None

    @property
    def has_other(self):
        """Whether the 'Other' option is enabled."""
        return self.raw.get('showOtherItem', self.raw.get('hasOther', False))

    @property
    def other_text(self):
        return self.raw.get('otherText', 'Other')

    @property
    def has_none(self):
        """Whether the 'None' option is enabled."""
        return self.raw.get('showNoneItem', self.raw.get('hasNone', False))

    @property
    def choices_order(self):
        return self.raw.get('choicesOrder', 'none')

    @property
    def col_count(self):
        return self.raw.get('colCount', 0)
